#! /usr/bin/env python

import os, os.path, pwd, readline, re, ConfigParser
from socket import gethostname
from getpass import getuser
from time import strftime

from conf import datadir


param={}


def data_file(fname):
    return os.path.join(datadir,fname)

def canonicalize(str):
    return re.sub("[^A-Za-z0-9@]","_",str)

def try_config(config,key):
    if key not in param and config.has_option("skel",key):
        param[key]=config.get("skel",key)

def set_defaults():
    config = ConfigParser.ConfigParser()
    config.read([data_file('defaults'),
                 os.path.expanduser('~/.gnomeskelrc'),
                 '.gnomeskelrc'])

    try_config(config,"AUTHOR")
    if "AUTHOR" not in param:
        pwent=pwd.getpwuid(os.getuid())
        param["AUTHOR"]=pwent[4].split(",")[0]

    try_config(config,"EMAIL")
    if "EMAIL" not in param and "EMAIL" in os.environ:
        param["EMAIL"]=os.environ["EMAIL"]
    if "EMAIL" not in param:
        param["EMAIL"]=getuser()+"@"+gethostname()

    if "DATE" not in param:
        param["DATE"]=strftime("%x")
    if "YEAR" not in param:
        param["YEAR"]=strftime("%Y")
    
    try_config(config,"BASEDIR")
    if "BASDIR" not in param:
        param["BASEDIR"]=os.getcwd()

def set_defaults_derived():
    if not "TOPSOURCEDIR" in param:
        param["TOPSOURCEDIR"]=os.path.join(param["BASEDIR"],param["UNIXNAME"])
        if not "FULLNAME" in param:
            param["FULLNAME"]="".join(map(lambda x:x.capitalize(),
                                          param["UNIXNAME"].split("-")))
    param["CANONICALNAME"]=canonicalize(param["UNIXNAME"])
    param["CANONICALNAMEUPPERCASE"]=param["CANONICALNAME"].upper()

def ask_string(key):
    first="QUIET" not in param
    while first or key not in param:
        if key in param:
            default=param[key]
            readline.insert_text(default)
        else:
            default=""
        x=raw_input("%s [%s]: "%(key.lower(),default))
        if x: param[key]=x
        first=False

keyword=re.compile(r"@([A-Z]+)@")
def find_infile(fname):
    infile=os.path.join("tmpl",fname)
    if os.path.exists(infile): return infile
    return data_file(fname)
def repl_key(matchobj):
    return param[matchobj.group(1)]
def copy_in(fname,inputname=None,mode=0644):
    if inputname:
        infile=find_infile (inputname)
    else:
        infile=find_infile (os.path.basename(fname))
    outfile=os.path.join(param["TOPSOURCEDIR"],fname)
    outdir=os.path.dirname(outfile)
    if not os.path.exists(outdir): os.makedirs(outdir)
    fdin=open(infile,"r")
    fdout=open(outfile,"w")
    for l in fdin.xreadlines():
        m=re.sub(keyword, repl_key, l)
        m.replace("@@","@")
        fdout.write(m)
    fdin.close()
    fdout.close()
    os.chmod(outfile,mode)
